/*
 *	Sherlock Library -- Linked Lists
 *
 *	(c) 2003 Martin Mares <mj@ucw.cz>
 *
 *	This software may be freely distributed and used according to the terms
 *	of the GNU Lesser General Public License.
 */

#ifndef _SHERLOCK_LISTS_H
#define _SHERLOCK_LISTS_H

typedef struct node {
  struct node *next, *prev;
} node;

typedef struct list {
  struct node head;
} list;

static inline void *list_head(list *l)
{
  return (l->head.next != &l->head) ? l->head.next : NULL;
}

static inline void *list_tail(list *l)
{
  return (l->head.prev != &l->head) ? l->head.prev : NULL;
}

static inline void *list_next(list *l, node *n)
{
  return (n->next != &l->head) ? (void *) n->next : NULL;
}

static inline void *list_prev(list *l, node *n)
{
  return (n->prev != &l->head) ? (void *) n->prev : NULL;
}

#define WALK_LIST(n,list) for(n=(void*)(list).head.next; (node*)(n) != &(list).head; n=(void*)((node*)(n))->next)

static inline void list_insert(node *what, node *after)
{
  node *before = after->next;
  what->next = before;
  what->prev = before->prev;
  before->prev = what;
  after->next = what;
}

static inline void list_add_tail(list *l, node *n)
{
  list_insert(n, l->head.prev);
}

static inline void list_add_head(list *l, node *n)
{
  list_insert(n, &l->head);
}

static inline void list_remove(node *n)
{
  node *before = n->prev;
  node *after = n->next;
  before->next = after;
  after->prev = before;
}

static inline void list_init(list *l)
{
  node *head = &l->head;
  head->next = head->prev = head;
}

#endif
