/*
 *	Netgrind -- Histograms
 *
 *	(c) 2003 Martin Mares <mj@ucw.cz>
 *
 *	This software may be freely distributed and used according to the terms
 *	of the GNU General Public License.
 */

#include "lib/lib.h"
#include "netgrind/pkt.h"
#include "netgrind/netgrind.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

static FILE *hist_file;
static list hist_list;
static uns hist_bucket_start, hist_threshold;
static uns hist_bucket_size = 60;

struct hist_entry {
  node n;
  byte *name;
  enum {
    HE_STATS,
    HE_INT
  } type;
  union {
    struct {
      struct pkt_stats *var;
      struct pkt_stats last_state;
    } st;
    struct {
      int *var;
      int last_state;
    } in;
  } u;
};

void histogram_init(byte *name)
{
  hist_file = fopen(name, "w");
  if (!hist_file)
    die("Unable to open %s: %m", name);
  list_init(&hist_list);
}

void histogram_add_stat(byte *name, struct pkt_stats *stat)
{
  struct hist_entry *e = xmalloc_zero(sizeof(*e));
  list_add_tail(&hist_list, &e->n);
  e->name = name;
  e->type = HE_STATS;
  e->u.st.var = stat;
}

void histogram_add_int(byte *name, int *var)
{
  struct hist_entry *e = xmalloc_zero(sizeof(*e));
  list_add_tail(&hist_list, &e->n);
  e->name = name;
  e->type = HE_INT;
  e->u.in.var = var;
}

static void hist_header(void)
{
  struct hist_entry *e;
  fprintf(hist_file, "# time");
  WALK_LIST(e, hist_list)
    fprintf(hist_file, "\t%s", e->name);
  fputc('\n', hist_file);
}

static void hist_print(void)
{
  struct hist_entry *e;
  fprintf(hist_file, "%d", hist_bucket_start);
  WALK_LIST(e, hist_list)
    switch (e->type)
      {
      case HE_STATS:
	fprintf(hist_file, "\t%Ld(%Ld)",
		e->u.st.var->packets - e->u.st.last_state.packets,
		e->u.st.var->bytes - e->u.st.last_state.bytes);
	break;
      case HE_INT:
	fprintf(hist_file, "\t%d", *e->u.in.var - e->u.in.last_state);
	break;
      default:
	ASSERT(0);
      }
  fputc('\n', hist_file);
}

void histogram_step(uns time)
{
  if (!hist_file || time < hist_threshold)
    return;
  if (!hist_bucket_start)
    hist_header();
  else
    hist_print();
  hist_bucket_start = time - (time % hist_bucket_size);
  hist_threshold = hist_bucket_start + hist_bucket_size;

  struct hist_entry *e;
  WALK_LIST(e, hist_list)
    switch (e->type)
      {
      case HE_STATS:
	e->u.st.last_state = *e->u.st.var;
	break;
      case HE_INT:
	e->u.in.last_state = *e->u.in.var;
	break;
      default:
	ASSERT(0);
      }
}

void histogram_cleanup(void)
{
  if (hist_file)
    {
      if (hist_threshold)
	hist_print();
      fclose(hist_file);
    }
}
