/*
 *	Netgrind -- IP Layer Analyser
 *
 *	(c) 2003 Martin Mares <mj@ucw.cz>
 *
 *	This software may be freely distributed and used according to the terms
 *	of the GNU General Public License.
 */

#include "lib/lib.h"
#include "netgrind/pkt.h"
#include "netgrind/netgrind.h"

#include <stdio.h>
#include <netinet/in.h>
#include <netinet/ip.h>

uns tcpip_calc_checksum(void *data, uns len, uns csum)
{
  /* FIXME: This is awfully slow and it probably consumes most of our run time */
  byte *x = data;

  while (len >= 2)
    {
      csum += (x[0] << 8) | x[1];
      if (csum & 0xffff0000)
	{
	  csum &= 0x0000ffff;
	  csum++;
	}
      x += 2;
      len -= 2;
    }
  if (len)
    {
      csum += x[0] << 8;
      if (csum & 0xffff0000)
	{
	  csum &= 0x0000ffff;
	  csum++;
	}
    }
  return csum;
}

uns tcpip_verify_checksum(uns csum)
{
  return (csum == 0xffff);
}

struct pkt_stats stat_ip_in, stat_ip_invalid, stat_ip_uninteresting, stat_ip_fragmented, stat_ip_badsum;

void ip_got_packet(struct pkt *p)
{
  struct iphdr *iph;

  pkt_account(&stat_ip_in, p);
  if (!(iph = pkt_peek(p, sizeof(*iph))))
    goto invalid;
  if (iph->ihl < 5)
    goto invalid;
  if (iph->version != 4)
    goto invalid;
  uns hdrlen = 4*iph->ihl;
  if (pkt_len(p) < hdrlen)
    goto invalid;
  if (!tcpip_verify_checksum(tcpip_calc_checksum(p->data, hdrlen, 0)))
    {
      pkt_account(&stat_ip_badsum, p);
      goto drop;
    }
  uns len = ntohs(iph->tot_len);
  if (len < hdrlen || len > pkt_len(p))
    goto invalid;
  pkt_unappend(p, pkt_len(p) - len);
  pkt_pop(p, hdrlen);

  if (iph->protocol != IPPROTO_TCP)
    {
      pkt_account(&stat_ip_uninteresting, p);
      goto drop;
    }
  /* XXX: Fragmentation not supported yet, but well-behaved TCP stacks don't use it anyway */
  if (ntohs(iph->frag_off) & 0x3fff)
    {
      pkt_account(&stat_ip_fragmented, p);
      goto drop;
    }
  tcp_got_packet(iph, p);
  return;

 invalid:
  pkt_account(&stat_ip_invalid, p);
 drop:
  pkt_free(p);
}
