/*
 *	Netgrind -- Packet Buffers
 *
 *	(c) 2003 Martin Mares <mj@ucw.cz>
 *
 *	This software may be freely distributed and used according to the terms
 *	of the GNU General Public License.
 */

#include "lib/lists.h"

struct pkt {
  node n;
  u64 timestamp;
  u32 seq;
  byte *data, *stop, *ebuf;
  byte buf[0];
};

static inline uns pkt_len(struct pkt *p)
{
  return p->stop - p->data;
}

static inline void pkt_push(struct pkt *p, uns len)
{
  p->data -= len;
  ASSERT(p->data >= p->buf);
}

static inline void *pkt_pop(struct pkt *p, uns len)
{
  byte *d = p->data;
  if (d + len > p->stop)
    return NULL;
  p->data += len;
  return d;
}

static inline void *pkt_peek(struct pkt *p, uns len)
{
  byte *d = p->data;
  if (d + len > p->stop)
    return NULL;
  return d;
}

static inline byte *pkt_append(struct pkt *p, uns len)
{
  byte *d = p->stop;
  p->stop = d + len;
  ASSERT(p->stop <= p->ebuf);
  return d;
}

static inline byte *pkt_unappend(struct pkt *p, uns len)
{
  p->stop -= len;
  ASSERT(p->stop >= p->data);
  return p->stop;
}

struct pkt *pkt_new(uns preroom, uns postroom);
void pkt_free(struct pkt *pkt);
void pkt_flush_queue(list *l);

struct pkt_stats {
  u64 packets;
  u64 bytes;
};

static inline void pkt_account(struct pkt_stats *stats, struct pkt *pkt)
{
  stats->packets++;
  stats->bytes += pkt_len(pkt);
}
